pls_one_model = function(title="no_title",X, y, a_final, scale=FALSE,
                         PDFfile="") {

# Make one ('final') PLS model from X and y with a_final components
#  and using all objects as calibration set (no optimization, no CV).

# A list with the model parameters is produced. 

#  ARGUMENTS
#  =========

#  title         text string with max. 20 characters (appears in outputs)

#  X             numerical matrix (n objects=rows, m variables=columns)
#                    X must not contain constant variables

#  y             dependent variable (length(y)=n) 
  
#  a_final       no. of PLS components to be used 
#                (determined previously, e. g., by rdCV)
#                (value not checked!)

#  scale          autoscaling of X if TRUE

#  PDFfile        if not "" a PDF with diagnostic plots is made 

 
#  VALUES
#  ======

#  A list 'pls_model':

#      title       title (as input or reduced to first 20 characters)
#      b0          intercept
#      b           regression coefficients (vector with m elements)
#      Xscale      = TRUE if X austoscaled before PLS, else FALSE
#      Xmean       arithmetic means of x-variables for model application
#                   (vector with m elements), 0's if no autoscaling !

#      Xsd         standard deviations of x-variables for model application
#                    (vector with m elements), 1's if no autoscaling ! 
#      SEC         standard error of calibration
#                  = sd of prediction errors (within calibration set) 
#      bias_calib  mean of prediction errors (within calibration set)
#      R2calib     squared Pearson corr. coeff. between predicted y and
#                   experimental (input) y 
#      n           no. of objects (in X)
#      m           no. of variables (in X)
#      X           X as in input (not autoscaled), also for variable names!
#      y           y as in input
#      y_fit       y predicted by this model 
#      a_final     no. PLS components (=input)                         
#      PLSmode     character string describing the PLS method (e. g. "simpls") 
#      origin      version of this function and date 

version = "pls_one_model 130226 VK"     # K. Varmuza, TU Vienna
comptime = proc.time()

library(pls)

n = nrow(X)
m = ncol(X)
ny = length(y)
title_red = substr(title,1,20)    # take only first 20 characters of title

#Xscaled = scale(X,center=TRUE,scale=TRUE)  # requires all variables with sd>0 !!

cat("\nStart: ",version)
cat("\nTitle:",title_red)
cat("\nX:",n,"x",m,"   y:",ny,"values")
if(scale) {
   cat("\n Autoscale X before PLS")
   XX = scale(X,center=TRUE,scale=TRUE)
   Xmean = apply(X,2,mean)
   Xsd = apply(X,2,sd)    }
else {
   cat("\n No scaling of X before PLS")
   XX = X
   Xmean = rep(0,m)
   Xsd = rep(1,m)   }

PLSmode = "simpls"
r = plsr(y~XX,ncomp=a_final,scale=FALSE,method=PLSmode,validation="none")
##############

cat("\nPLS with all objects and",a_final,"components made: ")

SEC = sd(r$residuals[,1,a_final])
bias_calib = mean(r$residuals[,1,a_final])
res_min = min(r$residuals[,1,a_final])
res_max = max(r$residuals[,1,a_final])
y_fit = r$fitted.values[,1,a_final]
R2calib = (cor(y_fit, y) )**2
cat("\n SEC =",SEC,"   R2calib =",R2calib)
cat("\n Calibration prediction errors:",res_min,"to",res_max,
       "\n             mean (bias_calib):",bias_calib) 

b0= r$Ymeans                      # intercept
b = r$coefficients[,1,a_final]    # regr. coeff.
cat("\n Intercept (b0) =",b0)
b_summary=summary(b)
cat("\n Regression coeff.:",b_summary[1],"to",b_summary[6],
      " mean =",b_summary[4])


# PDF with diagnostic plots 
# -------------------------

if (PDFfile != "") {
#~~~~~~~~~~~~~~~~~~~~~~

 pdf(file=PDFfile,paper="special",pagecentre=TRUE,width=6.5,height=10.6)
 par(mfrow = c(3,2))        # 3 rows each with 2 plots  
 cat("\nPDFfile ",PDFfile, " opened, ")

 source("text2PDF.R")
 
# Text box 1 (left)
# -----------------

n_lines = 11           # no. of lines in following text output
t=rep("",n_lines)  
t[1] = paste("Title: ",title_red)
t[2] = "* PLS model 'final' *"
t[3] = " "
t[4] = paste("X:   n =",n,"  m =",m)
t[5] = paste("y:", round(min(y),3),"to",round(max(y),3),"  sd =",
                   round(sd(y),3))
t[6] = " "
t[7] = paste("Auto scaling X =",scale)
t[8] = paste("No. PLS components =",a_final)
t[9] = paste("PLS method =",PLSmode)
t[10] = " "
t[11] = PDFfile

text2PDF(t,width=8,height=6.5,line_height=0.5,box=FALSE)

# Text box 2 (right)
# ------------------

n_lines = 11
t=rep("",n_lines) 
t[1] = date()
t[2] = paste("R-function: ",version)
t[3] = "  "
t[4] = paste("SEC =",round(SEC,3))
t[5] = paste("bias (calib) =",round(bias_calib,3))
t[6] = paste("Residuals =",round(res_min,3),"to",round(res_max,3))
t[7] = paste("R2 (y, y_fit) =", round(R2calib,4))
t[8] = " "
t[9] = paste("b0 =",round(b0,4))
t[10] = paste("b =",round(min(b),4),"to",round(max(b),4))
t[11] = " "

text2PDF(t,width=8,height=6.5,line_height=0.5,box=FALSE)


 # Plot 1: Predicted (fitted) y versus experimental y
# ---------------------------------------------------

    ylimits = range(y_fit)
    plot(y, y_fit, main="y-fitted versus y-experimental",
        xlab = "Experimental y", ylab = "Fitted y",
        cex.lab = 1.2, cex = 0.7, pch = 3, 
        ylim = ylimits)
    abline(c(0,1), lty = 1)

# Plot 2: Residuals versus object number 
# --------------------------------------

    resid = y - y_fit 
    ylimits <- max(abs(resid))
    ylimits <- sort(c(-ylimits, ylimits))
    plot(1:n, resid, main="Residuals of objects",
         xlab = "Object number", ylab = "Residuals (fitted)",
         cex.lab = 1.2, cex = 0.7, pch = 3, ylim = ylimits)
    abline(h = 0, lty = 1)

# Plot 3: Density distribution of residuals 
# -----------------------------------------

    plot(density(resid), main="Distribution of residuals",
         cex.lab = 1.2, cex = 0.7, pch = 3, col="red",
         xlab="Residuals (Quantiles 2.5%, 97.5%)",ylab="Density")
    abline(h=0,lwd=0.7)
    abline(v = 0,lwd=0.7)
    abline(v=quantile(resid,0.025),lty=2,lwd=0.7)
    abline(v=quantile(resid,0.9755),lty=2,lwd=0.7)

# Plot 4: Regression coefficient plot 
# ------------------------
     plot(b,type="h",main="Regression coefficients (b)", xlab="Variable number",
          ylab="b", lwd=1, col="blue")
     abline(h=0,lwd=0.7)

dev.off()
cat("closed.")

 }  # end if PDFfile ~~~~~~~~~~~~~~~~~~~~~~

comptime = proc.time() - comptime
cat("\nEnd of ",version, "   ",comptime[1],"s \n")

pls_model = list (
                title = title_red,
                b0 = b0,
                b = b,
                Xscale = scale,
                Xmean = Xmean,
                Xsd = Xsd,
                SEC = SEC,
                bias_calib = bias_calib,
                R2calib = R2calib,
                n = n,
                m = m,
                X = X,
                y = y,
                y_fit = y_fit,
                a_final = a_final,
                PLSmode = PLSmode,
                origin = paste(version,date())  )       

return(pls_model)
 
}  # end function
