varsel_corr_xy = function(X, y, method="pearson", 
                          m_sel=0, corr_limit=0.7, messages=TRUE) {

# Variable selection: maximum correlation measure with y

# Correlation measure is the squared correlation coefficient calculated
# by one of the methods Pearson, Spearman, or Kendall.

# The (maximum) number of the selected variables and/or the minimum value of the 
# correlation measure can be defined (see examples below).

#  ARGUMENTS
#  =========

#  X              numerical matrix (n objects=rows, m variables=columns)

#  y              numerical vector for correlation (length(y)=n) 

#  method        "pearson" or "spearman" or "kendall"
  
#  m_sel          max. no. of variables selected having corr. measure >= cor_limit
#                    m_sel=0 means all variables with corr.measure >= corr_limit

#  corr_limit     variables selected have corr. measure >= corr_limit

#                 Example: select all variables with corr. measure > 0.7
#                          m_sel = m (or m_sel <= 0)
#                          corr_limit =  0.7

#                 Example: select 10 variables with highest corr. measure
#                          m_sel = 10
#                          corr_limit =  0

#                 Example: select a maximum of 5 variables with highest
#                          corr. measure >= c0.8
#                          m_sel = 5
#                          corr_limit = 0.8 

#  messages       messages on screen if TRUE
 
#  VALUES
#  ======

#  var_corr_xy   result vector (logical, m components)
#                  TRUE   variable selected
#                  FALSE  variable deleted     

#  Input parameters are checked as follows:
#       lenght(y) == nrow(X)
#       m_sel = 0 | ( m_sel < ncol(X) & m_sel >= 1)
#       corr_limit <= 1 & corr_limit >= 0        
#  STOP with a message if at least one of them gives FALSE
                       
version = "varsel_corr_xy 130222 VK"     # K. Varmuza, TU Vienna

n = nrow(X)
m = ncol(X)
if (messages) {
  cat("\nStart: ",version)
  cat("\nDelete variables (columns in X) with too low corr. measure to y",
      "\nMethod:",method,
      "\nMax no. of selected variables =",m_sel,"(0=all)",
      "\nMin correlation measure (squared corr. coeff.) with y =",corr_limit)
  cat("\nX(",n,"x",m,")")
}

# Check input

input_error = FALSE
k = 0                      # summed input error code
if (length(y) != n)              {input_error=TRUE; k = k+1}   
if ((m_sel != 0) & (m_sel > m))  {input_error=TRUE; k = k+2}
if ((m_sel != 0) & (m_sel <=0))  {input_error=TRUE; k = k+4}   
if (corr_limit > 1)              {input_error=TRUE; k = k+8}
if (corr_limit < 0)              {input_error=TRUE; k = k+16}
if (input_error) {stop("ERROR in varsel_corr_xy: input parameter wrong,
                        code =",k,"\n") }  

sel = rep(TRUE,m)        # selected variables init.

r = cor(X,y, method=method)    # corrr. coeff.
r = r**2                       # squared, element-wise
rmin=min(r)
rmax=max(r)

# Mark variables FALSE if correlation too low

for (j in 1:m) {
  if(r[j] < corr_limit) {sel[j]= FALSE}  }
mj = sum(sel)
if(messages) {cat("\n",(m-mj),"variables deleted with corr.measure <",corr_limit,
                   "   ",mj,"variables remaining")}

# Consider max. no. of variables

sel_ready = FALSE

if (m_sel==0 | m_sel==m) {sel_ready = TRUE}   # all variables with sufficient
                                              #  correlation
                                                                            
if (!sel_ready & (m_sel>=mj)) {sel_ready = TRUE}   # no. of variables with 
                                                    # sufficient correlation
                                                    # is <= m_sel (take all)

if (!sel_ready & (m_sel<mj) ){   # select among the remaining mj variables
                                 # a subset with m_sel variables possessing
                                 # highest correlations
  
  rsel = cbind(r,seq(1:m))   # matrix: 1st col = r
                             #         2nd col = index (1 ... m)
  rsel = rsel[sel,]          # only variables selected by corr. measure
  
  n_rsel=nrow(rsel)
  m_rsel=ncol(rsel)
  x_sort = sort(rsel[,1],index.return=TRUE,decreasing=TRUE)
                             # sort by col 1 (= r) and make index vector
  rsel_sort=rsel[x_sort$ix,] # matrix with r and original variable numbers
                             #  sorted by decreasing r (indices: x_sort$ix) 
  
  # Mark variables at row (m_sel+1) to last row as being FALSE
  for (j in (m_sel+1):mj) {sel[rsel_sort[j,2]]=FALSE}
 }                                                                      
  
mj = sum(sel)     # no. of remaining variables
if (messages) {
    cat("\nFinal result of variable selection:",
        "\n",(m-mj),"variables deleted, ",mj," variables remain. End. \n") }

var_corr_xy = sel      # logical result vector (selected variables)
return(var_corr_xy)
 
}  # end function
