varsel_pls_regr_coeff = function(X, y, m_sel = 2, regr_coeff_limit = c(0,0), 
                                 a_to_use = 0,
                                 rdCV_param = c(10,50,3,5,1), 
                                 PDFfile="", messages=TRUE) {

# Variable selection: by absolute standardized regression coefficients (b)
#                     in a rdCV-optimized PLS model from all objects

# The (maximum) number of selected variables and/or the minimum value of the 
# absolute correlation coefficients can be defined (see examples below).

#  ARGUMENTS
#  =========

#  X              numerical matrix (n objects=rows, m variables=columns)
#                    X must not contain constant variables (sd <> 0)

#  y              dependent variable (length(y)=n) 
  
#  m_sel          max. no. of variables selected with regr.coeff. (b) in the
#                              defined ranges (see 'regr_coeff_limit')
#                 m_sel=0 (or m_sel=m) means all variables with b in the 
#                                      allowed ranges
#
#  regr_coeff_limit  = vector with 2 elements, e.g. =c(-2, 0.5)
#                      regr_coeff_limit[1]  lower limit, for negative b,
#                       variable deleted if b between 0 and regr_coeff_limit[1] 
#                      regr_coeff_limit[2]  higher limit, for positive b,
#                       variable deleted if b between 0 and regr_coeff_limit[2]

#                      Variables with b in the interval (exclusive)
#                          regr_coeff_limit[1] to regr_coeff_limit[2]
#                          are deleted .
#                      Values 0 and 0 are used for 'no limits'.

#        Example: select all variables with ABS(b) > 0.3
#                            m_sel = m (or m_sel = 0)
#                            regr_coeff_limit = c(-0.3, 0.3)

#        Example: select 10 variables with the highest ABS(b)
#                            m_sel = 10
#                            regr_coeff_limit = c(0,0)  no limits

#        Example: select a maximum of 5 variables with the 
#                 positive b >= 0.3, and negative b <= -1
#                            m_sel = 5
#                            regr_coeff_limit = c(-1, 0.3)

#        Example: select a maximum of 5 variables with positive b >= 0.3, 
#                  and no variables with a negative b
#                          m_sel = 5
#                          regr_coeff_limit = c(-1e30, 0.3) neg. values all not

#  a_to_use     no. of PLS components to be used
#               <=0   estimated by rdCV
#               else  no rdCV, a_to_use is used for a_final (for 'final model')
#               However: a_final = min(n, m, a_to_use)
 
#  rdCV_param   rdCV parameter  (details see go_rdcv function)

#               rdCV_param [1]   amax (max. desired no. of PLS comp.)
#               rdCV_param [2]   repetitions
#               rdCV_param [3]   seg_test (no. of segments in outer CV loop,
#                                          random split into calibration
#                                          sets and test sets)
#               rdCV_param [4]  seg_calib (no. of segments in inner CV loop,
#                                          CV with random split for optimizing
#                                          the no. of PLS components)
#               rdCV_param [5]  parsimony_factor  (for 'standard error 
#                                           method' used to optimize the no.
#                                           of PLS components)

#  PDFfile      if not "" a PDF with this name with diagnostic plots is made 

#  messages     messages on screen if TRUE
 
#  VALUES
#  ======

#  var_pls_regr_coeff   result vector (logical, m components)
#                        TRUE   variable selected
#                        FALSE  variable deleted     
                 
version = "varsel_pls_regr_coeff 130222 VK"     # K. Varmuza, TU Vienna

comptime = proc.time()

n = nrow(X)
m = ncol(X)
Xscaled = scale(X,center=TRUE,scale=TRUE)  # requires all variables with sd>0 !!

if (messages) {
  cat("\nStart: ",version)
  cat("\nDelete variables (columns in X) with too low absolute regr.",
         "coeff. (b) \n   in rdCV-optimized PLS model",
      "\nMax no. of selected variables =",m_sel,"(0=all)",
      "\nLimits for b: for a negative b the variable is deleted if b >",
                                                          regr_coeff_limit[1],
      "\n              for a positive b the variable is deleted if b <",
                                                          regr_coeff_limit[2]) 
  if(a_to_use <= 0) { # estimate a_final by rdCV
     cat("\nrdCV parameter: amax =",rdCV_param[1]," repetitions =",rdCV_param[2],
      "\n                seg_test =",rdCV_param[3]," seg_calib =",rdCV_param[4],
                         "  parsimonity =",rdCV_param[5]) }
  else {             # no rdCV                        
      a_final = min(n,m,a_to_use)
      cat("\nNo rdCV because a_final given =",a_to_use,
          "\n                a_final used  =",a_final)  }

  cat("\nX(",n,"x",m,")")
}

ymin = min(y)
ymax = max(y)
ysd = sd(y)

# Check input
# -----------
#  Input parameters are checked as follows:
#   lenght(y)==nrow(X)
#   m_sel < 0) | m_sel > m
#   regr_coeff_limit[1] <= 0
#   regr_coeff_limit[2] >= 0
#   (m_sel=0 | m_sel=m) & regr_coeff_limit[1]=-1e30 & regr_coeff_limit[2]=1e30        
#  STOP with a message if at least one of these checks gives FALSE

input_error = FALSE
k = 0                      # summed input error code
if (length(y) != n)              {input_error=TRUE; k = k+1}   
if ((m_sel < 0) | (m_sel > m))   {input_error=TRUE; k = k+2}
if (regr_coeff_limit[1] > 0)     {input_error=TRUE; k = k+4}
if (regr_coeff_limit[2] < 0)     {input_error=TRUE; k = k+8}
if (((m_sel==0) | (m_sel==m)) & regr_coeff_limit[1]==-1e30 & 
      regr_coeff_limit[2]==1e30) {input_error=TRUE; k = k+16}
if (input_error) {stop("ERROR in varsel__pls_regr_coeff: input parameter(s)",
                       " wrong, code =",k,"\n") }  

# Check whether rdCV or not
# -------------------------

if (a_to_use <= 0) {  

source("go_rdcv.R")   # includes library(chemometrics, pls)
  
res = go_rdcv(title="var_sel",
            Xscaled, y,
            method="simpls",
            amax=rdCV_param[1],
            repetitions=rdCV_param[2],
            seg_test=rdCV_param[3],
            seg_calib=rdCV_param[4],
            parsimony_factor=rdCV_param[5],
            PDFfile="",
            messages=0)

a_final = res$a_final
SEP = res$SEP_final
cat("\nrdCV results: a_final =",a_final," with SEP =",SEP)

}  # end if (a_to_use <= 0) {  # check whether rdCV or not
           
# Make final model: PLS with a_final components, with Xscale (autoscaled)
# ----------------

res_pls = plsr(y~Xscaled, ncomp=a_final, scale=FALSE, method="simpls",
               validation="none")

cat("\nPLS with all autoscaled X-data and",a_final,"components yields: ")

SEC = sd(res_pls$residuals[,1,a_final])
bias_calib = mean(res_pls$residuals[,1,a_final])
res_min = min(res_pls$residuals[,1,a_final])
res_max = max(res_pls$residuals[,1,a_final]) 
cat("SEC =",SEC)
if(messages){
   cat("\nCalibration prediction errors:",res_min,"to",res_max,
       "\n            mean (bias_calib):",bias_calib) }

b =  res_pls$coefficients[,1,a_final]   # (standardized) regr. coeff.

b_summary=summary(b)
if(messages) {
 cat("\nRegression coeff. (from autoscaled X):",b_summary[1],"to",b_summary[6],
      " mean =",b_summary[4])
 cat("\n Quantiles 0.1, 0.2, 0.8, 0.9 =",quantile(b,c(0.1, 0.2, 0.8, 0.9))) 
}

# Mark variables FALSE if b outside defined ranges

sel = rep(TRUE,m)        # selected variables init.

for (j in 1:m) {
  if((b[j] < 0) & b[j] > regr_coeff_limit[1]) {sel[j]= FALSE}  
  if((b[j] >=0) & b[j] < regr_coeff_limit[2]) {sel[j]= FALSE}  
  }
mj = sum(sel)
if(messages) {cat("\n",(m-mj),"variables deleted with b outside defined ranges,",
              mj,"variables remaining")}

# Consider max. no. of variables

sel_ready = FALSE

if (m_sel==0 | m_sel==m |(mj <= m_sel)) {sel_ready = TRUE} 
                      # take all variables with good b
                      # (no restriction for m_sel or mj <= m_sel)
 
  else  {      # no. (mj) of variables with good b is larger than m_sel
               # t.m. select a subset with m_sel variables with 
               # highest ABS(b), thus positive and negative b are treated in
               # the same way and perhaps different limits are not considered!

  bsel = cbind(abs(b),seq(1:m))  # matrix: 1st col = absolute b
                                 #         2nd col = variable index (1 ... m)
  bsel = bsel[sel,]              # only variables with good b

  x_sort = sort(bsel[,1],index.return=TRUE,decreasing=TRUE)
                             # sort by col 1 (= ABS(b)) and make index vector
  bsel_sort=bsel[x_sort$ix,] # matrix with b_abs and original variable numbers
                             #   sorted by decreasing b_abs (indices: x_sort$ix) 
  
  # Mark variables (in sel) given at row (m_sel+1) to last row (of bsel_sort)
  #  as FALSE (these are the variables more than m_sel) 
  
  for (j in (m_sel+1):mj) {sel[bsel_sort[j,2]]=FALSE }
 
 } # end: if (!sel_ready & (m_sel<mj) )  

mj = sum(sel)     # no. of remaining variables
cat("\nFinal result of variable selection:",
        "\n",(m-mj),"variables deleted, ",mj," variables remain. End.")

var_pls_regr_coeff = sel      # logical result vector (selected variables)


# PDF with diagnostic plots
# -------------------------

if (PDFfile != "") {
#~~~~~~~~~~~~~~~~~~~~~~

 pdf(file=PDFfile,paper="special",pagecentre=TRUE,width=6.5,height=10.6)
 par(mfrow = c(3,2))        # 3 rows each with 2 plots  
 cat("\nPDFfile ",PDFfile, " opened, ")

 source("text2PDF.R")
 
# Text box  (left)
# ----------------

n_lines = 12           # no. of lines in following text output
t=rep("",n_lines)  
t[1] = paste("Variable selection: abs(PLS-regress.coeff.)")
t[2] = paste("R-fct: ",version)
t[3] = paste("X:   n =",n,"  m =",m)
t[4] = paste("y:", round(ymin,3),"to",round(ymax,3),"  sd =",
                   round(ysd,3))

if (a_to_use <= 0) { 
t[5] = paste("rdCV: amax/r/seg-test/calib/parsim=")
t[6] = paste("     ",rdCV_param[1],rdCV_param[2],rdCV_param[3],rdCV_param[4],
                     rdCV_param[5])
}
else {
t[5] = paste("No rdCV: a_final desired=",a_to_use)
t[6] = " "
}

t[7] = paste("m_sel (max) = ",m_sel,"   b-limits: ",regr_coeff_limit[1],"...",
             regr_coeff_limit[2],sep="")
t[8] = paste("m_sel (result) = ",mj)             
t[9] = paste("Regr. coeff. b=",b_summary[1],"...",b_summary[6])             


if (a_to_use <= 0) { 
t[10] = paste("a_final=",a_final,"  SEP=",round(SEP,3),"  SEC=",round(SEC,3),sep="")
}
else {
t[10] = paste("a_final=",a_final,"  SEP=NA","  SEC=",round(SEC,3),sep="")
}

t[12] = PDFfile

text2PDF(t,width=8,height=6.5,line_height=0.5,box=FALSE)


if (a_to_use <= 0) {  

# Plot: Frequencies of optimum a-values (no. of PLS comp.)

   plot(table(res$a_opt_all), 
     main=paste("Frequencies of a_opt (",rdCV_param[2]*rdCV_param[3],
                " values)",sep=""), cex.main=1,
     xlab=paste("no. PLS comp. (a_final=",a_final,")",sep=""),
     ylab="Frequency",lwd=5)

# Plot: y-predicted (rdCV, test set objects) versus y

    preddcvall <- res$ypred_all[,a_final,] # all rdCV predcited y at a_final
    predmean <- apply(preddcvall,1,mean)   # mean of repetitions
    R2predmean = cor(predmean,y)^2
    ylimits = range(preddcvall)
    plot(y, predmean, 
        main=paste("rdCV:  SEP = ",round(SEP,2),"   R2 = ",
                                  round(R2predmean,4),sep=""),
        cex.main=1,
        xlab = "Experimental y",  
        ylab = "Test set predicted y",
        cex.lab = 1.2, cex = 0.9, pch = 3, col = gray(0.9), type = "n",
        ylim = ylimits)
    for (i in 1:ncol(preddcvall)) {
        points(y, preddcvall[, i], cex = 0.3, pch = 3,
            col = gray(0.6))
    }
    points(y,predmean, cex = 0.7, pch = 3, col = "blue")
    abline(c(0,1), lty = 1)

} # end if (a_to_use <= 0)

# Plot: y-predicted (calibration) versus y

    yfit = res_pls$fitted.values[,1,a_final]
    R2fit = cor(yfit,y)^2
    ylimits = range(yfit)
    plot(y, yfit, 
        main=paste("Calibration:  SEC = ",round(SEC,2),"   R2 = ",
                                  round(R2fit,4),sep=""),
        cex.main=1,
        xlab = "Experimental y",  
        ylab = "Calibration set (all) predicted y",
        cex.lab = 1.2, cex = 0.9, pch = 3, col = gray(0.6), type = "n",
        ylim = ylimits)
    points(y,yfit, cex = 0.9, pch = 3, col = "red")
    abline(c(0,1), lty = 1)

 
 par(mfrow = c(3,1))   # one column plot (next page)
 
 # Plot: regression coefficients
 
 plot(b,type="h", col="white",
       main="Standardized regression coefficients b (PLS model, rdCV-optimized)",
       xlab=paste("Variable number (1 to",m,")      ",
                   mj,"variables selected in red (max", m_sel,
                  "), excluded b-range:",regr_coeff_limit[1],"to",
                  regr_coeff_limit[2] ))
 for (j in 1:m) {  # all bars
   if(sel[j]) {colorbar = "red"}
         else {colorbar = gray(0.6)} 
    lines(x=c(j,j),y=c(0,b[j]),col=colorbar)
  }  
 
 abline(h=0)  # horiz. axis
 if(regr_coeff_limit[1]>=min(b) & regr_coeff_limit[1] != 0)
   {abline(h=regr_coeff_limit[1],lty="dashed",col="blue")}
 if(regr_coeff_limit[2]<=max(b) & regr_coeff_limit[2] != 0)
   {abline(h=regr_coeff_limit[2],lty="dashed",col="green")}  
 
dev.off()
cat("closed.")

 }  # end if PDFfile ~~~~~~~~~~~~~~~~~~~~~~

comptime = proc.time() - comptime
cat("\nEnd of ",version, "   ",comptime[1],"s \n")

return(var_pls_regr_coeff)
 
}  # end function
