varsel_stepwise_BIC = function(X, y, mode="both", maxTime=200, maxsteps=20,
                               r_step_resultfile="r_step_BIC.RData",
                               messages=TRUE) {

# Variable selection: stepwise forward or forward/backward ("both")
#  using the BIC criterion, as implemented in the functions 'stepforward' 
#  and 'stepboth' 121009, PF,  both included here as 'external functions'.

#  Ref.: Heide Garcia and Peter Filzmoser, vignette for chemometrics' (2010),
#  Multivariate Statistical Analysis using the R package chemometrics

#  Software: Peter Filzmoser and Leonhard Seyfang, TU Vienna  

#  The algorithm starts with the empty univariate model where the dependent
#  variable y is explained only by the intercept, and adds or removes in each
#  step one variable until no more improvement can be done or
#  until a certain number of steps or a given maximum computing time is reached. 

#  The criterion used for the decision which variable to add is the 
#  Bayesian information criterion (BIC).

#  The original R-function 'step' is not used because it is very slow
#  for ca >200 variables.

# Packages used: pls


#  ARGUMENTS
#  =========

#  X              numerical matrix (n objects=rows, m variables=columns)

#  y              numerical vector for correlation, length(y)=n

#  mode           "forward" or "both" (stepwise in both directions) 

#  maxTime        max. computing time (seconds)
#                 (time is checked after each step !)
                  
#  maxstep        max. no. of steps

#  resultfile     if NOT "": result (a list) from 'stepforward' or 'stepboth'
#                 is written to this RData-file (as object 'r_step') 

#  messages       messages on screen if TRUE
 
#  VALUES
#  ======

#  var_stepwise_BIC   result vector (logical, m components)
#                      TRUE   variable selected
#                      FALSE  variable deleted     
                       
version = "varsel_stepwise_BIC3 130222 VK"        # K. Varmuza, TU Vienna

comptime = proc.time()
library(pls)

n = nrow(X)
m = ncol(X)
ny = length(y)
if (messages) {
  cat("\nStart: ",version)
  cat("\nStepwise (forward/backward) selection of variables (columns in X).",
      "\nMethod as implemented in 'stepforward' and 'stepboth'",
      "criterion = BIC")
  cat("\n  Mode =",mode, "  max comp. time =",maxTime,"  max no. of steps =",maxsteps)    
  cat("\nX(",n,"x",m,")  y:",ny)
}
 
Xy = data.frame(X=X, y=y)

if (mode=="forward") {
       r = stepforward(y~., data=Xy, maxTime=maxTime, maxsteps=maxsteps) }
           #----------
else {
       r = stepboth(y~., data=Xy, maxTime=maxTime, maxsteps=maxsteps) }
           #-------

steps_done = nrow(r$model)
var_stepwise_BIC=r$model[steps_done,]==TRUE    
                                    # logical vector with selected variables

mj = sum(var_stepwise_BIC)                      # no. of selected variables
seconds <- r$usedTime[steps_done]               # computing time
  

cat("\nResult of stepwise variable selection:",steps_done,"steps net, ",
         seconds,"s",  
        "\n",(m-mj),"variables deleted, ",mj,"variables selected. \n")

if (r_step_resultfile != "") {
    r_step = r
    save(r_step,file=r_step_resultfile)
    cat("\nResults from stepwise selection written to file:",r_step_resultfile)
}

if (messages) {         
   plot(r$bic,type="l",xlab="step",ylab="BIC") }

if (messages) {
comptime = proc.time() - comptime
cat("\nEnd of ",version, "   ",comptime[1],"s \n")   }

return(var_stepwise_BIC)
 
}  # end function

#######################################################################
#  Local functions ('stepforward' and 'stepboth', P. Filzmoser, 121009)
#  Some messages (marked by #***) included by VK 121120
#######################################################################

stepforward <-
function (formula, data, k, maxTime = 1800, 
    writeFile = FALSE, resname = "stepres00", maxsteps = 500, 
    ...) 
{
    cat("\n*** Start stepforward. Step no. (time): \n")        #***
    require(pls)
    mf <<- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data"), names(mf), 0)
    mf <- mf[c(1, m)]
    mf[[1]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    mt <- attr(mf, "terms")
    y <- model.response(mf, "numeric")
    y <- as.matrix(y)
    colnames(y) <- deparse(formula[[2]])
    x <- pls:::delete.intercept(model.matrix(mt, mf))
    x <- as.data.frame(x)
    form <- function(a, namesX) {
        if (sum(a) == 0) 
            return(as.formula("y~1"))
        as.formula(paste("y~", paste(namesX[as.logical(a)], collapse = "+"), 
            sep = ""))
    }
    startTime <- as.integer(Sys.time())
    if (missing(k)) 
        k <- log(nrow(x))
    dat <- data.frame(y, x)
    p <- ncol(x)
    namesX <- names(x)
    bic <- rep(Inf, maxsteps)
    usedTime <- rep(NA, maxsteps)
    models <- matrix(NA, maxsteps, p)
    avec <- rep(0,p) # for formula, corresponds to empty model
    for (i in 1:maxsteps) {
        #print(i)                                   #***
        nowTime = as.integer(Sys.time())- startTime #***
        cat(i,"(",round(nowTime,0),") ")            #***
        if ((i-(i%/%5)*5)==0) {cat("\n")}           #***
        addv <- which(avec==0) # these variables can be added
        bici <- rep(Inf,p)
        for (j in addv){ 
          modelj <- avec
          modelj[j] <- 1
          fmlj <- form(modelj,namesX)
          bici[j] <- AIC(lm(fmlj,data=dat),k=k)
        }
        jopt <- which.min(bici)
        usedTime[i] <- as.integer(Sys.time()) - startTime
        bic[i] <- bici[jopt]
        avec[jopt] <- 1
        models[i, ] <- avec
        if (i >= 2) {
            if (bic[i] >= bic[i - 1]) 
                break
        }
        if (as.integer(Sys.time()) - startTime >= maxTime) 
            break
     #***   try(write.table(cbind(usedTime, bic, models), file = paste(resname, 
     #***       ".txt", sep = ""), col.names = FALSE, row.names = FALSE))
    }
    if (any(is.na(models[, 1]))) {
        tmp <- which(is.na(models[, 1]))[1] - 1
        bic <- bic[1:tmp]
        usedTime <- usedTime[1:tmp]
        models <- models[1:tmp, ]
    }
    if (length(bic) > 1) {
        if (bic[length(bic)] >= bic[length(bic) - 1]) {
            tmp <- length(bic) - 1
            bic <- bic[1:tmp]
            usedTime <- usedTime[1:tmp]
            models <- models[1:tmp, ]
        }
    }
    if (writeFile) {
        write.table(cbind(usedTime, bic, models), file = paste(resname, 
            ".txt", sep = ""), col.names = FALSE, row.names = FALSE)
    }
    return(list(usedTime = usedTime, bic = bic, models = models))
}

#################

stepboth <-
function (formula, data, k, maxTime = 1800, direction = "both", 
    writeFile = FALSE, resname = "stepres00", maxsteps = 500, 
    ...) 
{
    cat("\n*** Start stepboth. Step no. (time): \n")         #***
    require(pls)
    mf <<- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data"), names(mf), 0)
    mf <- mf[c(1, m)]
    mf[[1]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    mt <- attr(mf, "terms")
    y <- model.response(mf, "numeric")
    y <- as.matrix(y)
    colnames(y) <- deparse(formula[[2]])
    x <- pls:::delete.intercept(model.matrix(mt, mf))
    x <- as.data.frame(x)
    form <- function(a, namesX) {
        if (sum(a) == 0) 
            return(as.formula("y~1"))
        as.formula(paste("y~", paste(namesX[as.logical(a)], collapse = "+"), 
            sep = ""))
    }
    startTime <- as.integer(Sys.time())
    if (missing(k)) 
        k <- log(nrow(x))
    dat <- data.frame(y, x)
    p <- ncol(x)
    namesX <- names(x)
    bic <- rep(Inf, maxsteps)
    usedTime <- rep(NA, maxsteps)
    models <- matrix(NA, maxsteps, p)
    avec <- rep(0,p) # for formula, corresponds to empty model
    for (i in 1:maxsteps) {
        #print(i)                                   #***
        nowTime = as.integer(Sys.time())- startTime #***
        cat(i,"(",round(nowTime,0),") ")            #***
        if ((i-(i%/%5)*5)==0) {cat("\n")}           #***
        # backward selection
        if (sum(avec)>0){         # possibility to go back
          addv <- which(avec==1)  # these variables can be deleted
          bici <- rep(Inf,p)
          for (j in addv){
            modelj <- avec
            modelj[j] <- 0
            fmlj <- form(modelj,namesX)
            bici[j] <- AIC(lm(fmlj,data=dat),k=k)
          }
          joptback <- which.min(bici)
          bicback <- bici[joptback]
        }
        else { # not able to go back
          bicback <- Inf
        }
        # forward selection
        addv <- which(avec==0) # these variables can be added
        bici <- rep(Inf,p)
        for (j in addv){ 
          modelj <- avec
          modelj[j] <- 1
          fmlj <- form(modelj,namesX)
          bici[j] <- AIC(lm(fmlj,data=dat),k=k)
        }
        joptforw <- which.min(bici)
        bicforw <- bici[joptforw]
        
        # select optimal strategy
        if (bicback<bicforw){ # backward is better
          bic[i] <- bicback
          avec[joptback] <- 0
        }
        else { # forward is better
          bic[i] <- bicforw
          avec[joptforw] <- 1
        }
        usedTime[i] <- as.integer(Sys.time()) - startTime
        models[i, ] <- avec
        if (i >= 2) {
            if (bic[i] >= bic[i - 1]) 
                break
        }
        if (as.integer(Sys.time()) - startTime >= maxTime) 
            break
    #***    try(write.table(cbind(usedTime, bic, models), file = paste(resname, 
    #***        ".txt", sep = ""), col.names = FALSE, row.names = FALSE))
    }
    if (any(is.na(models[, 1]))) {
        tmp <- which(is.na(models[, 1]))[1] - 1
        bic <- bic[1:tmp]
        usedTime <- usedTime[1:tmp]
        models <- models[1:tmp, ]
    }
    if (length(bic) > 1) {
        if (bic[length(bic)] >= bic[length(bic) - 1]) {
            tmp <- length(bic) - 1
            bic <- bic[1:tmp]
            usedTime <- usedTime[1:tmp]
            models <- models[1:tmp, ]
        }
    }
    if (writeFile) {
        write.table(cbind(usedTime, bic, models), file = paste(resname, 
            ".txt", sep = ""), col.names = FALSE, row.names = FALSE)
    }
    return(list(usedTime = usedTime, bic = bic, models = models))
}

#  very end


